'**************************************************************************
'* Superpro.VB
'*
'* (C) Copyright 1991-2002 Rainbow Technologies, Inc. All rights reserved.
'*
'* Description - SuperPro file provide defination and wrappers for SSP dual Library
'*
'****************************************************************************/
Imports System.Runtime.InteropServices
Imports System.Text
Imports System
Imports Microsoft.VisualBasic

Module SuperproLib
    Public Class Superpro
        'Error Code for SSP library
        Public Const SP_SUCCESS As Integer = 0
        Public Const SP_INVALID_FUNCTION_CODE As Integer = 1
        Public Const SP_INVALID_PACKET As Integer = 2
        Public Const SP_UNIT_NOT_FOUND As Integer = 3
        Public Const SP_ACCESS_DENIED As Integer = 4
        Public Const SP_INVALID_MEMORY_ADDRESS As Integer = 5
        Public Const SP_INVALID_ACCESS_CODE As Integer = 6
        Public Const SP_PORT_IS_BUSY As Integer = 7
        Public Const SP_WRITE_NOT_READY As Integer = 8
        Public Const SP_NO_PORT_FOUND As Integer = 9
        Public Const SP_ALREADY_ZERO As Integer = 10
        Public Const SP_DRIVER_OPEN_ERROR As Integer = 11
        Public Const SP_DRIVER_NOT_INSTALLED As Integer = 12
        Public Const SP_IO_COMMUNICATIONS_ERROR As Integer = 13
        Public Const SP_PACKET_TOO_SMALL As Integer = 15
        Public Const SP_INVALID_PARAMETER As Integer = 16
        Public Const SP_MEM_ACCESS_ERROR As Integer = 17
        Public Const SP_VERSION_NOT_SUPPORTED As Integer = 18
        Public Const SP_OS_NOT_SUPPORTED As Integer = 19
        Public Const SP_QUERY_TOO_LONG As Integer = 20
        Public Const SP_INVALID_COMMAND As Integer = 21
        Public Const SP_MEM_ALIGNMENT_ERROR As Integer = 29
        Public Const SP_DRIVER_IS_BUSY As Integer = 30
        Public Const SP_PORT_ALLOCATION_FAILURE As Integer = 31
        Public Const SP_PORT_RELEASE_FAILURE As Integer = 32
        Public Const SP_ACQUIRE_PORT_TIMEOUT As Integer = 39
        Public Const SP_SIGNAL_NOT_SUPPORTED As Integer = 42
        Public Const SP_UNKNOWN_MACHINE As Integer = 44
        Public Const SP_SYS_API_ERROR As Integer = 45
        Public Const SP_UNIT_IS_BUSY As Integer = 46
        Public Const SP_INVALID_PORT_TYPE As Integer = 47
        Public Const SP_INVALID_MACH_TYPE As Integer = 48
        Public Const SP_INVALID_IRQ_MASK As Integer = 49
        Public Const SP_INVALID_CONT_METHOD As Integer = 50
        Public Const SP_INVALID_PORT_FLAGS As Integer = 51
        Public Const SP_INVALID_LOG_PORT_CFG As Integer = 52
        Public Const SP_INVALID_OS_TYPE As Integer = 53
        Public Const SP_INVALID_LOG_PORT_NUM As Integer = 54
        Public Const SP_INVALID_ROUTER_FLGS As Integer = 56
        Public Const SP_INIT_NOT_CALLED As Integer = 57
        Public Const SP_DRVR_TYPE_NOT_SUPPORTED As Integer = 58
        Public Const SP_FAIL_ON_DRIVER_COMM As Integer = 59
        Public Const SP_SERVER_PROBABLY_NOT_UP As Integer = 60
        Public Const SP_UNKNOWN_HOST As Integer = 61
        Public Const SP_SENDTO_FAILED As Integer = 62
        Public Const SP_SOCKET_CREATION_FAILED As Integer = 63
        Public Const SP_NORESOURCES As Integer = 64
        Public Const SP_BROADCAST_NOT_SUPPORTED As Integer = 65
        Public Const SP_BAD_SERVER_MESSAGE As Integer = 66
        Public Const SP_NO_SERVER_RUNNING As Integer = 67
        Public Const SP_NO_NETWORK As Integer = 68
        Public Const SP_NO_SERVER_RESPONSE As Integer = 69
        Public Const SP_NO_LICENSE_AVAILABLE As Integer = 70
        Public Const SP_INVALID_LICENSE As Integer = 71
        Public Const SP_INVALID_OPERATION As Integer = 72
        Public Const SP_BUFFER_TOO_SMALL As Integer = 73
        Public Const SP_INTERNAL_ERROR As Integer = 74
        Public Const SP_PACKET_ALREADY_INITIALIZED As Integer = 75
        Public Const SP_PROTOCOL_NOT_INSTALLED As Integer = 76

        'String for OS Driver type
        Public Const RB_WINNT_SYS_DRVR As Integer = 5                  ' Windows NT system driver
        Public Const RB_WIN95_SYS_DRVR As Integer = 7                      ' Windows 95 system driver
        Public Const RB_NW_LOCAL_DRVR As Integer = 8           ' Netware local driver

        '' Maximum number of devices 
        Public Const MAX_NUM_DEV As Integer = 10

        ' Maximum host name length 
        Public Const MAX_NAME_LEN As Integer = 64
        ' Maximum host address length 
        Public Const MAX_ADDR_LEN As Integer = 32

        ' Making the LIcense Update Time programmable
        Public Const MAX_HEARTBEAT As Integer = 2592000 ' 30*24*60*60 seconds 
        Public Const MIN_HEARTBEAT As Integer = 60 'RSC 60 seconds 
        Public Const INFINITE_HEARTBEAT As Integer = -1

        Public Const SPRO_MAX_QUERY_SIZE As Integer = 56

        'Communication Modes 
        Public Const RNBO_STANDALONE As String = "RNBO_STANDALONE"
        Public Const RNBO_SPN_DRIVER As String = "RNBO_SPN_DRIVER"
        Public Const RNBO_SPN_LOCAL As String = "RNBO_SPN_LOCAL"
        Public Const RNBO_SPN_BROADCAST As String = "RNBO_SPN_BROADCAST"
        Public Const RNBO_SPN_ALL_MODES As String = "RNBO_SPN_ALL_MODES"
        Public Const RNBO_SPN_SERVER_MODES As String = "RNBO_SPN_SERVER_MODES"

        ' api PACKET DEFINATION AND INITIALIZATION
        Public Const SPRO_APIPACKET_SIZE As Integer = 4112
        Public Shared packet As Byte() = New Byte(SPRO_APIPACKET_SIZE) {}

        '' Class for ServerInfo
        Public Class ServerInfo
            Public serverAddress() As Byte = New Byte(MAX_ADDR_LEN) {}
            Public numOfAvail As System.Int16

            'Class member function for getting server info from byte buffer passed to API
            'Buffer : variable passed to EnumServer API
            'offset : start index for byte buffer to get value 
            Public Sub GetServerInfo(ByVal buffer As Byte(), ByVal offset As Integer)
                System.Array.Clear(serverAddress, 0, MAX_ADDR_LEN)
                Dim j As Integer = 0
                Dim i As Integer = offset
                ' get first 32 byte from buffer starting from offset value
                Do
                    serverAddress(j) = buffer(i)
                    i = i + 1
                    j = j + 1
                Loop Until (i > MAX_ADDR_LEN + offset)
                ' get next 2 byte value for NumLic
                numOfAvail = BitConverter.ToInt16(buffer, MAX_ADDR_LEN + offset)
            End Sub

        End Class
        ' Structure for key info
        Public Structure KeyInfo
            Public developerId As System.Int16
            Public hardLimit As System.Int16
            Public inUse As System.Int16
            Public numTimeOut As System.Int16
            Public highestUse As System.Int16
        End Structure
        ' class for Monitor info
        Public Class MonitorInfo
            Public serverName() As Byte = New Byte(MAX_NAME_LEN) {}
            Public serverIPAddress() As Byte = New Byte(MAX_ADDR_LEN) {}
            Public serverIPXAddress() As Byte = New Byte(MAX_ADDR_LEN) {}
            Public version() As Byte = New Byte(MAX_NAME_LEN) {}
            Public protocol As System.Int16
            Public sproKeyMonitorInfo As KeyInfo = New KeyInfo()
            ' Member function for getting key info from byte buffer passed to API
            ' Buffer: passed to GetKeyInfo API
            Public Sub GetMonitorInfo(ByVal buffer As Byte())
                Dim index As Integer = 0
                Dim j As Integer = 0
                Dim offset As Integer = 0

                Do
                    serverName(j) = buffer(index)
                    j = j + 1
                    index = index + 1
                Loop Until index = MAX_NAME_LEN

                offset = index
                j = 0
                Do
                    serverIPAddress(j) = buffer(index)
                    j = j + 1
                    index = index + 1
                Loop Until index = (MAX_ADDR_LEN + offset)

                offset = index
                j = 0
                Do
                    serverIPXAddress(j) = buffer(index)
                    j = j + 1
                    index = index + 1
                Loop Until index = (MAX_ADDR_LEN + offset)

                offset = index
                j = 0
                Do
                    version(j) = buffer(index)
                    j = j + 1
                    index = index + 1
                Loop Until index = (MAX_NAME_LEN + offset)

                protocol = BitConverter.ToInt16(buffer, index)
                sproKeyMonitorInfo.developerId = BitConverter.ToInt16(buffer, index + 2)
                sproKeyMonitorInfo.hardLimit = BitConverter.ToInt16(buffer, index + 4)
                sproKeyMonitorInfo.inUse = BitConverter.ToInt16(buffer, index + 6)
                sproKeyMonitorInfo.numTimeOut = BitConverter.ToInt16(buffer, index + 8)
                sproKeyMonitorInfo.highestUse = BitConverter.ToInt16(buffer, index + 10)
            End Sub

        End Class

        Public Enum PROTOCOL_FLAG
            NSPRO_TCP_PROTOCOL = 1
            NSPRO_IPX_PROTOCOL = 2
            NSPRO_NETBEUI_PROTOCOL = 4
            NSPRO_SAP_PROTOCOL = 8
        End Enum

        Public Enum ENUM_SERVER_FLAG
            NSPRO_RET_ON_FIRST = 1
            NSPRO_GET_ALL_SERVERS = 2
            NSPRO_RET_ON_FIRST_AVAILABLE = 4
        End Enum

        Public Declare Ansi Function RNBOsproFormatPacket Lib "sx32w.dll" Alias _
        "RNBOsproFormatPacket" (ByVal packet As Byte(), _
                                ByRef ApiPackSize As Integer) As System.Int16

        Public Declare Ansi Function RNBOsproInitialize Lib "sx32w.dll" Alias _
        "RNBOsproInitialize" (ByVal packet As Byte()) As System.Int16

        Public Declare Ansi Function RNBOsproGetFullStatus Lib "sx32w.dll" Alias _
        "RNBOsproGetFullStatus" (ByVal packet As Byte()) As System.Int16

        Public Declare Ansi Function RNBOsproGetVersion Lib "sx32w.dll" Alias _
        "RNBOsproGetVersion" (ByVal packet As Byte(), _
                              ByRef majv As Byte, _
                              ByRef minv As Byte, _
                              ByRef rev As Byte, _
                              ByRef ostype As Byte) As System.Int16

        Public Declare Ansi Function RNBOsproFindFirstUnit Lib "sx32w.dll" Alias _
        "RNBOsproFindFirstUnit" (ByVal packet As Byte(), _
                                 ByVal developerID As Int32) As System.Int16

        Public Declare Ansi Function RNBOsproFindNextUnit Lib "sx32w.dll" Alias _
        "RNBOsproFindNextUnit" (ByVal packet As Byte()) As System.Int16

        Public Declare Ansi Function RNBOsproRead Lib "sx32w.dll" Alias _
        "RNBOsproRead" (ByVal packet As Byte(), _
                        ByVal address As Integer, _
                        ByRef data As Int32) As System.Int16

        Public Declare Ansi Function RNBOsproExtendedRead Lib "sx32w.dll" Alias _
        "RNBOsproExtendedRead" (ByVal packet As Byte(), _
                                ByVal address As Integer, _
                                ByRef data As Int32, _
                                ByRef accessCode As Byte) As System.Int16

        Public Declare Ansi Function RNBOsproWrite Lib "sx32w.dll" Alias _
        "RNBOsproWrite" (ByVal packet As Byte(), _
                         ByVal wPass As Int32, _
                         ByVal address As Int16, _
                         ByVal data As Int32, _
                         ByVal accessCode As Byte) As System.Int16

        Public Declare Ansi Function RNBOsproOverwrite Lib "sx32w.dll" Alias _
        "RNBOsproOverwrite" (ByVal packet As Byte(), _
                             ByVal wPass As Int32, _
                             ByVal oPass1 As Int32, _
                             ByVal oPass2 As Int32, _
                             ByVal address As Int16, _
                             ByVal datum As Int32, _
                             ByVal accessCode As Byte) As System.Int16

        Public Declare Ansi Function RNBOsproDecrement Lib "sx32w.dll" Alias _
        "RNBOsproDecrement" (ByVal packet As Byte(), _
                             ByVal wPass As Int32, _
                             ByVal address As Int32) As System.Int16

        Public Declare Ansi Function RNBOsproActivate Lib "sx32w.dll" Alias _
        "RNBOsproActivate" (ByVal packet As Byte(), _
                            ByVal wPass As Int32, _
                            ByVal aPass1 As Int32, _
                            ByVal aPass2 As Int32, _
                            ByVal address As Int16) As System.Int16

        Public Declare Ansi Function RNBOsproQuery Lib "sx32w.dll" Alias _
        "RNBOsproQuery" (ByVal packet As Byte(), _
                         ByVal address As Int16, _
                         ByVal queryString As Byte(), _
                         ByVal response As Byte(), _
                         ByRef response32 As Long, _
                         ByVal length As Int16) As System.Int16

        Public Declare Ansi Function RNBOsproGetHardLimit Lib "sx32w.dll" Alias _
        "RNBOsproGetHardLimit" (ByVal packet As Byte(), _
                                ByRef hardlimit As Int32) As System.Int16

        Public Declare Ansi Function RNBOsproGetContactServer Lib "sx32w.dll" Alias _
        "RNBOsproGetContactServer" (ByVal packet As Byte(), _
                                    ByVal serverName As Byte(), _
                                    ByVal bufferSize As Integer) As System.Int16

        Public Declare Ansi Function RNBOsproSetContactServer Lib "sx32w.dll" Alias _
        "RNBOsproSetContactServer" (ByVal packet As Byte(), _
                                    ByVal serverName As String) As System.Int16

        Public Declare Ansi Function RNBOsproGetSubLicense Lib "sx32w.dll" Alias _
        "RNBOsproGetSubLicense" (ByVal packet As Byte(), _
                                 ByVal cellAddress As Int16) As System.Int16

        Public Declare Ansi Function RNBOsproSetProtocol Lib "sx32w.dll" Alias _
        "RNBOsproSetProtocol" (ByVal packet As Byte(), _
                               ByVal protocolFlag As Int16) As System.Int16

        Public Declare Ansi Function RNBOsproSetHeartBeat Lib "sx32w.dll" Alias _
        "RNBOsproSetHeartBeat" (ByVal packet As Byte(), _
                                ByVal heartbeat As Int32) As System.Int16

        Public Declare Ansi Function RNBOsproReleaseLicense Lib "sx32w.dll" Alias _
        "RNBOsproReleaseLicense" (ByVal packet As Byte(), _
                                  ByVal cellAddress As Int16, _
                                  ByRef numOfLic As Int32) As System.Int16

        Public Declare Ansi Function RNBOsproEnumServer Lib "sx32w.dll" Alias _
        "RNBOsproEnumServer" (ByVal enumflag As Integer, _
                              ByVal developerid As Int32, _
                              ByVal serverBuffer As Byte(), _
                              ByRef numOfServers As Int16) As System.Int16

        Public Declare Ansi Function RNBOsproGetKeyInfo Lib "sx32w.dll" Alias _
        "RNBOsproGetKeyInfo" (ByVal packet As Byte(), _
                              ByVal developerId As Int32, _
                              ByVal keyIndex As Int16, _
                              ByVal keyBuffer As Byte()) As System.Int16

    End Class
End Module
